﻿''' <summary>
''' 色チップ
''' </summary>
''' <remarks>[Label]コントロールを継承する</remarks>
Public Class ColorChip

    Inherits Label '表面的には[Label]コントロールを装う。



    '============================================================================================



    ''' <summary>
    ''' 色の明るさ判定に用いる閾値
    ''' </summary>
    ''' <remarks></remarks>
    Private ReadOnly BrightnessBorder As Double = Distance3D_P2P(0, 0, 0, 255, 255, 255) * 0.56 '0.56の意味：0.5=中間, 0.06=調整値



    '============================================================================================



    Dim _SpecimenColor As Color
    Dim _Selected As Boolean = False


    '--------------------------------------------------------------------------------------------


    ''' <summary>
    ''' 標本色
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property SpecimenColor As Color

        Get
            Return _SpecimenColor
        End Get

        Set(value As Color)

            _SpecimenColor = value

            With Me

                .ForeColor = IIf((Nearness(Color.Black) < BrightnessBorder) OrElse (_SpecimenColor.A < &HFF \ 2), Color.White, Color.Black) '2点間の距離方式（やっぱりこれが一番しっくり来るなぁ。）
                '.ForeColor = IIf((_SpecimenColor.GetBrightness < 0.49) OrElse (_SpecimenColor.A < &HFF \ 2), Color.White, Color.Black) 'HSB(HSV).Brightness方式
                '.ForeColor = IIf((_SpecimenColor.R * 0.299 + _SpecimenColor.G * 0.587 + _SpecimenColor.B * 0.114 < 256 \ 2) OrElse (_SpecimenColor.A < &HFF \ 2), Color.White, Color.Black) '白黒方式
                .BackColor = _SpecimenColor
                .Text = _SpecimenColor.Name

            End With

        End Set

    End Property

    ''' <summary>
    ''' 選択中フラグ
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Selected As Boolean

        Get
            Return _Selected
        End Get

        Set(value As Boolean)

            _Selected = value

            If _Selected Then
                Me.Height *= 2
            Else
                Me.Height \= 2
            End If

        End Set

    End Property



    '============================================================================================



    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New(InitalColor As Color)

        SpecimenColor = InitalColor

        With Me
            .Name = InitalColor.Name
            .BorderStyle = BorderStyle.FixedSingle
            .Margin = New Padding(2)
            .Padding = New Padding(1)
            .Size = New Size(100, 23)
            .Cursor = Cursors.Hand
            .AutoEllipsis = True
        End With

    End Sub

    ''' <summary>
    ''' 色の近さ
    ''' </summary>
    ''' <param name="Cardinal">基点となる色</param>
    ''' <returns>基点色からの距離</returns>
    ''' <remarks></remarks>
    Public Function Nearness(Cardinal As Color) As Double

        Return Distance3D_P2P(Cardinal.R, Cardinal.G, Cardinal.B, SpecimenColor.R, SpecimenColor.G, SpecimenColor.B)

    End Function

    ''' <summary>
    ''' 3次元での2点間の距離
    ''' </summary>
    ''' <param name="x1">点1のx座標</param>
    ''' <param name="y1">点1のy座標</param>
    ''' <param name="z1">点1のz座標</param>
    ''' <param name="x2">点2のx座標</param>
    ''' <param name="y2">点2のy座標</param>
    ''' <param name="z2">点2のz座標</param>
    ''' <returns>距離</returns>
    ''' <remarks></remarks>
    Private Function Distance3D_P2P(ByVal x1 As Integer, ByVal y1 As Integer, ByVal z1 As Integer, ByVal x2 As Integer, ByVal y2 As Integer, ByVal z2 As Integer) As Double

        Return Math.Sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2 + (z2 - z1) ^ 2)

    End Function

End Class
