﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『《ComboBox》,《ListBox》の独自描画について』
'  │・「色リスト」や「覚え書き」に付いている小さな色見本は、《DrawMode》プロパティの変更と《ItemDraw》イベントによる描画で、実現している。
'  │・《DrawMode》プロパティを指定すると、そこに表示される各アイテムの高さが、フォントの高さではなく《ItemHeight》プロパティに支配されるようにり、フォントを大きくすると下部が切れてしまうことになる。
'  │・そこで、初期設定時に、《ItemHeight》プロパティにコントロールの《Font.Height》プロパティの値を代入することにした。
'  │・参考：http://shinshu.fm/MHz/88.44/archives/0000030651.html, http://homepage1.nifty.com/rucio/main/dotnet/Samples/Sample112ListBoxWithIcon.htm
'  │
'  │『《ComboBox》,《ListBox》への色チップの登録について』
'  │・《DisplayMember》,《ValueMember》,《DataSource》プロパティを利用することで、任意のオブジェクトのコレクションを選択項目として利用できるようになる。
'  │・参考：http://homepage1.nifty.com/rucio/main/dotnet/Samples/Sample114ListBoxItemData.htm, http://dobon.net/vb/dotnet/control/tbdisplaymember.html
'  │
'  │『近似色の照合について』
'  │・色の近さは、RGB色空間（RGB値をXYZ座標の3次元空間とみなす）で2点間の距離を求めて比較する。他にもHSB(HSV),LABといった色空間で探す方法もあるみたい。RGB色空間の方法は簡単な代わりに淡い色の扱いが苦手（ピンクは赤より茶に近いと判定してしまうなど）という欠点があるけど、HSBなら解決できるらしい。
'  │・照合結果のソートについては、ソート用のクラスを用意して、記憶しておいた近似値の小さい順に並べ替えている。
'  │・参考：http://homepage1.nifty.com/rucio/main/dotnet/Samples/Sample108ListBoxSort.htm
'  │
'  │『《ToolStripMenuItem.DropDownItems》に《ToolStripLabel》を加える方法について』
'  │・フォーム デザイナでは《ToolStripMenuItem.DropDownItems》に直接《ToolStripLabel》を作製できないので、親である《ToolStrip》コントロールにいったん作製し、それをカット＆ペーストで追加しました。
'  └────────────





''' <summary>
''' 本体
''' </summary>
''' <remarks>メインのフォーム</remarks>
Public Class FormMain

    ''' <summary>改行を表す文字列</summary>
    Private Const ENTER_STRING As String = "|↵|"

    ''' <summary>色箱のマージン</summary>
    Private Const COLOR_BOX_MARGIN As Integer = 2

    ''' <summary>色箱の横幅</summary>
    Private Const COLOR_BOX_WIDTH As Integer = 8


    '--------------------------------------------------------------------------------------------


    ''' <summary>覚え書きの変更フラグ</summary>
    Private MemoChange As Boolean = False



    '============================================================================================



    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        '=== フォーム ===
        With My.Application.Info
            Me.Text = .Title

            lnkInformation.Text = "Version " & .Version.ToString & vbCrLf & .Copyright & vbCrLf & .CompanyName
            lnkInformation.Links.Add(lnkInformation.Text.IndexOf(.CompanyName), .CompanyName.Length, "http://www.h3.dion.ne.jp/~u-tyann0/digitalian_works/")
        End With

        tslEditArea.Text &= vbCrLf & "　{0} 改行　{1} 名前　{2} HTML名　{3} ARGB" & vbCrLf & "　{4} 透明度　{5} 赤　{6} 緑　{7} 青　{8} 色相(HLS)　{9} 彩度(HLS)　{10} 輝度(明度:HLS)"

        '=== 色目録 ===
        '定義色     ← システム カラー（《KnownColor》列挙体）
        'システム色 ← 定義色の内、Windowsの表示要素の色（《SystemColors》クラスのプロパティとして定義されている色）
        'Web色      ← 定義色の内、システム色以外の色（《Color》構造体のプロパティとして定義されている色）

        '--- 色リストの基本設定 ---
        With cmbDefinitionColor
            .DisplayMember = "Text"
            .ItemHeight = .Font.Height 'アイテムの高さ調整
        End With

        With cmbWebColor
            .DisplayMember = "Text"
            .ItemHeight = .Font.Height 'アイテムの高さ調整
        End With

        With cmbSystemColor
            .DisplayMember = "Text"
            .ItemHeight = .Font.Height 'アイテムの高さ調整
        End With

        '--- 色コレクションとWeb・システム色リスト作製 ---
        flpColorCollection.SuspendLayout() '┬ レイアウトロジックを一時停止する。これだけで高速化！ といってもこの規模じゃ1xxミリ秒ほどやけど。
        cmbWebColor.BeginUpdate()          '│
        cmbSystemColor.BeginUpdate()       '┘

        For Each colorName As String In [Enum].GetNames(GetType(KnownColor)) '《KnownColor》列挙体（全ての定義色が含まれる）のメンバ名を配列化（備忘録：《[Enum].GetNames(～)を[Enum].GetValues(～)》にしたら、値を配列化できる）。

            Dim newChip As ColorChip = New ColorChip(Color.FromName(colorName)) '色名から色チップを作る。
            Dim section As String = "" '所属先

            '色コレクションへ登録
            flpColorCollection.Controls.Add(newChip)
            AddHandler newChip.Click, AddressOf ColorChip_Click

            'Web色リストまたはシステム色リストに振り分け（と《DataSource》で登録しない場合は、《ValueMember》を使えない・・・今回は使わないけど、参考までに書いておく。）
            If newChip.SpecimenColor.IsSystemColor Then
                cmbSystemColor.Items.Add(newChip)
                section = "システム"
            Else
                cmbWebColor.Items.Add(newChip)
                section = "Web"
            End If

            'ヒントを付けてみる
            ttpHint.SetToolTip(newChip, String.Format("[{1}]{0}Dec:{2:D}{0}Hex:{2:X8}", vbCrLf, section, newChip.SpecimenColor.ToArgb))

        Next

        flpColorCollection.ResumeLayout() '┬ レイアウトロジックを再開する。
        cmbWebColor.EndUpdate()           '│
        cmbSystemColor.EndUpdate()        '┘

        '--- 色コレクションを定義色リストへ登録 ---
        cmbDefinitionColor.DataSource = flpColorCollection.Controls

        '=== 覚え書き ===
        lstMemo.ItemHeight = lstMemo.Font.Height 'アイテムの高さ調整
        MemoLoad()

        '=== ユーザ書式 ===
        PresetLoad()

        '=== パレット初期化 ===
        picSampleBox.BackColor = Color.Black '背景色
        nudNearRange.Maximum = Math.Ceiling(DirectCast(flpColorCollection.Controls("White"), ColorChip).Nearness(Color.Black)) '近似色照合の最大閾値（黒と白の距離）

        ColorReplace(Color.White)

    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing

        If MemoChange Then

            '=== 覚え書きを保存するか？ ===
            Select Case MessageBox.Show("「覚え書き」が、変更されています。" & vbCrLf & "保存しますか？", Me.Text, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Information)

                Case MsgBoxResult.Yes

                    '--- [はい] ---
                    Try
                        MemoSave()
                    Catch ex As Exception
                        e.Cancel = True '終了を中断する。
                    End Try

                Case MsgBoxResult.Cancel

                    '--- [キャンセル] ---
                    e.Cancel = True

            End Select

        End If

    End Sub

    Private Sub lnkInformation_LinkClicked(sender As System.Object, e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles lnkInformation.LinkClicked

        If MessageBox.Show("既定のブラウザで公式ウェブサイトを表示します。" & vbCrLf & e.Link.LinkData, Me.Text, MessageBoxButtons.OKCancel, MessageBoxIcon.Information, MessageBoxDefaultButton.Button2) = Windows.Forms.DialogResult.OK Then

            '=== Web Siteを開く ===
            Try
                System.Diagnostics.Process.Start(e.Link.LinkData)
                e.Link.Visited = True '訪問済み
            Catch ex As Exception
                MessageBox.Show("表示に失敗しました。" & vbCrLf & vbCrLf & ex.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
            End Try

        End If

    End Sub

    Private Sub cmbColorList_DrawItem(sender As System.Object, e As System.Windows.Forms.DrawItemEventArgs) Handles cmbDefinitionColor.DrawItem, cmbWebColor.DrawItem, cmbSystemColor.DrawItem

        '=== 色リスト（色箱付き《ComboBox》コントロール）のアイテム描画 ===
        If e.Index < 0 Then Exit Sub '※※※ アイテムがないときは何もしない。

        Dim chip As ColorChip = DirectCast(DirectCast(sender, ComboBox).Items(e.Index), ColorChip) '色チップ

        DrawColorBoxItem(chip.Text, chip.SpecimenColor, e)

    End Sub

    Private Sub lstMatchingResult_DrawItem(sender As Object, e As System.Windows.Forms.DrawItemEventArgs) Handles lstMatchingResult.DrawItem

        '=== 照合結果（色箱付き《ListBox》コントロール）のアイテム描画 ===
        If e.Index < 0 Then Exit Sub '※※※ アイテムがないときは何もしない。

        With DirectCast(DirectCast(sender, ListBox).Items(e.Index), ColorBrotherItem)

            '--- 照合結果に合わせた表示形式 ---
            Dim tempEventArgs As Windows.Forms.DrawItemEventArgs
            If .NearnessValue = 0 OrElse .NearnessValue > nudNearRange.Value Then
                tempEventArgs = New Windows.Forms.DrawItemEventArgs(e.Graphics, e.Font, e.Bounds, e.Index, e.State, e.BackColor, e.ForeColor)
            Else
                tempEventArgs = e
            End If

            '--- 描画 ---
            DrawColorBoxItem(String.Format("({0,3:##0}){2}{1}", Math.Round(.NearnessValue, MidpointRounding.AwayFromZero), .ColorName, IIf(.NearnessValue = 0, "*", " ")), Color.FromName(.ColorName), tempEventArgs)

        End With

        '注意：《ToString》（または、《DisplayMember》で指定したプロパティ）とは違う文字列を表示した場合、末尾がコントロール幅からはみ出したとしても、横スクロールバーは出現しないみたい。

    End Sub

    Private Sub lstMemo_DrawItem(sender As System.Object, e As System.Windows.Forms.DrawItemEventArgs) Handles lstMemo.DrawItem

        '=== 覚え書き（色箱付き《ListBox》コントロール）のアイテム描画 ===
        If e.Index < 0 Then Exit Sub '※※※ アイテムがないときは何もしない。

        Dim textString As String = DirectCast(sender, ListBox).Items(e.Index).ToString '表示する文字列
        Dim boxColor As Color = Color.FromArgb(Convert.ToInt32(textString, 16)) '表示する色

        DrawColorBoxItem(textString, boxColor, e)

    End Sub

    Private Sub cmbColorList_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbDefinitionColor.SelectedIndexChanged, cmbWebColor.SelectedIndexChanged, cmbSystemColor.SelectedIndexChanged

        '=== 色リストから選択 ===
        With DirectCast(sender, ComboBox)
            If .SelectedIndex > -1 Then ColorReplace(DirectCast(.SelectedItem, ColorChip).SpecimenColor)
        End With

    End Sub

    Private Sub lstMatchingResult_SelectedIndexChanged(sender As Object, e As System.EventArgs) Handles lstMatchingResult.SelectedIndexChanged

        '=== 照合結果から選択 ===
        With DirectCast(sender, ListBox)
            If .SelectedIndex > -1 Then ColorReplace(Color.FromName(DirectCast(.SelectedItem, ColorBrotherItem).ColorName))
        End With

    End Sub

    Private Sub ColorChip_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        '色チップのクリックイベントに割り当てられる。

        '=== 色チップから選択 ===
        ColorReplace(DirectCast(sender, ColorChip).SpecimenColor)

    End Sub

    Private Sub lstMemo_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstMemo.SelectedIndexChanged

        '=== 覚え書きの色をパレットに再現 ===
        With DirectCast(sender, ListBox)
            If .SelectedIndex > -1 Then
                ColorReplace(Color.FromArgb(Integer.Parse(.Text, Globalization.NumberStyles.HexNumber)))
                btnUp.Enabled = (.SelectedIndex > 0)
                btnDown.Enabled = (.SelectedIndex < .Items.Count - 1)
                btnDelete.Enabled = True
            Else
                btnUp.Enabled = False
                btnDown.Enabled = False
                btnDelete.Enabled = False
            End If
        End With

    End Sub

    Private Sub btnMemorize_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMemorize.Click

        '=== 覚え書き登録 ===
        With lstMemo

            Dim colorCode As String = DirectCast(picSampleBox.Tag, Color).ToArgb.ToString("X8")

            If .Items.Contains(colorCode) Then
                lstMemo.Text = colorCode '既に登録済み
            Else
                Try
                    .Items.Add(colorCode)
                    .TopIndex = .Items.Count - 1
                    MemoChanged()
                Catch ex As SystemException
                    DirectCast(sender, Button).Enabled = False
                    MessageBox.Show("これ以上記憶できません。", Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
                Catch ex As Exception
                    MessageBox.Show("予期しない例外が発生しました。" & vbCrLf & vbCrLf & ex.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
                End Try
            End If

        End With

    End Sub

    Private Sub btnDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDelete.Click

        '=== 覚え書き削除 ===
        If lstMemo.SelectedIndex > -1 Then
            lstMemo.Items.RemoveAt(lstMemo.SelectedIndex)
            MemoChanged()
            btnMemorize.Enabled = True
        End If

    End Sub

    Private Sub btnMemoItemMove_Click(sender As System.Object, e As System.EventArgs) Handles btnUp.Click, btnDown.Click

        '=== 覚え書きアイテムを上下に移動 ===
        With lstMemo

            Dim index As Integer = .SelectedIndex + CInt(DirectCast(sender, Button).Tag) '移動先の番号
            Dim item As Object = .SelectedItem '移動する項目

            .Items.Item(.SelectedIndex) = .Items.Item(index)
            .Items.Item(index) = item

            .SelectedIndex = index

            MemoChanged()

        End With

    End Sub

    Private Sub PalletElement_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles pltAlpha.ValueChanged, pltRed.ValueChanged, pltGreen.ValueChanged, pltBlue.ValueChanged

        '=== パレット要素の変更 ===
        ColorReplace(DirectCast(sender, PalletElement).ARGB)

    End Sub

    Private Sub BackColorSetting_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmBlack.Click, tsmWhite.Click, tsmPallet.Click

        '=== サンプルボックスの背景色設定 ===
        picSampleBox.BackColor = Choose(sender.tag, Color.Black, Color.White, DirectCast(picSampleBox.Tag, Color))

    End Sub

    Private Sub picSampleBox_BackColorChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles picSampleBox.BackColorChanged

        '=== パレット要素の背景色をサンプルボックスの背景色に合わせる ===
        With DirectCast(sender, PictureBox)

            pltAlpha.BackgroundColor = .BackColor
            pltRed.BackgroundColor = .BackColor
            pltGreen.BackgroundColor = .BackColor
            pltBlue.BackgroundColor = .BackColor

            ttpHint.SetToolTip(sender, String.Format("左クリック：合成色を取得{0}右クリック：背景色を変更{0}現在：{1}({2:X})", vbCrLf, .BackColor.Name, .BackColor.ToArgb))

        End With

    End Sub

    Private Sub picSampleBox_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles picSampleBox.Paint

        '=== サンプルボックス表示 ===
        With DirectCast(sender, PictureBox)
            e.Graphics.FillRectangle(New SolidBrush(DirectCast(.Tag, Color)), Rectangle.Inflate(.ClientRectangle, -4, -8))
        End With

    End Sub

    Private Sub picSampleBox_Click(sender As System.Object, e As System.EventArgs) Handles picSampleBox.Click

        '=== サンプルボックスの合成色（ベース色に背景色とパレット色を加味したもの）を取得 ===
        '※前提：ベース色（フォームの色）は、不透明であるとする。

        Dim a, r, g, b As Double

        '--- ベース色取得 ---
        With SystemColors.Control
            r = .R
            g = .G
            b = .B
        End With

        '--- 背景色・パレット色の合成 ---
        For Each blendColor As Color In New Color() {picSampleBox.BackColor, DirectCast(picSampleBox.Tag, Color)}

            With blendColor
                a = .A / 255 '合成比率
                r = r * (1 - a) + .R * a
                g = g * (1 - a) + .G * a
                b = b * (1 - a) + .B * a
            End With
            '適当に計算してみただけなので、これが正しいのかどうか自信ない。
            '《PictureBox》をBitmap化してピクセルの色を取得しようとも思ったけど、Windowsがフルカラーに設定されていなかった場合（タイルパターンで表示されるため）正しい色を取得できないから諦めた。

        Next

        ColorReplace(Color.FromArgb(255, r, g, b))

    End Sub

    Private Sub btnEditDialog_Click(sender As System.Object, e As System.EventArgs) Handles btnEditDialog.Click

        '=== Windows標準のカラーダイアログ（「色の設定」）で色を作る ===
        With cdlColorEditor

            Dim custom As New List(Of Integer)
            For i As Integer = 0 To 16
                If i = lstMemo.Items.Count Then Exit For
                custom.Add(ColorCodeReverses(Convert.ToInt32(lstMemo.Items(i), 16)))
            Next

            .Color = DirectCast(picSampleBox.Tag, Color) '初期色設定
            .CustomColors = custom.ToArray 'カスタム色

            If .ShowDialog = Windows.Forms.DialogResult.OK Then
                ColorReplace(.Color)
            End If

        End With

        '今回の開発で、HSVとかHLS色空間なるものを知れた。
        'そして、「色の設定」の右側がHLSだということも分かったし、使い方もなんとなく分かった。

    End Sub

    Private Sub btnReverse_Click(sender As System.Object, e As System.EventArgs) Handles btnReverse.Click

        '=== 反転色 ===
        With DirectCast(picSampleBox.Tag, Color)
            ColorReplace(Color.FromArgb(.A, &HFF - .R, &HFF - .G, &HFF - .B))
        End With

    End Sub

    Private Sub btnComplementary_Click(sender As System.Object, e As System.EventArgs) Handles btnComplementary.Click

        '=== 補色 ===
        '"Adobe Illustrator CS4"での補色計算法 ・・・ R,G,Bの内の最大値と最小値を足し、そこからR,G,Bを引く。
        'http://help.adobe.com/ja_JP/Illustrator/14.0/WS714a382cdf7d304e7e07d0100196cbc5f-6288a.html

        With DirectCast(picSampleBox.Tag, Color)
            Dim work As Integer = CInt(Math.Max(Math.Max(.R, .G), .B)) + CInt(Math.Min(Math.Min(.R, .G), .B))
            ColorReplace(Color.FromArgb(.A, work - .R, work - .G, work - .B))
        End With

    End Sub

    Private Sub btnMatching_Click(sender As System.Object, e As System.EventArgs) Handles btnMatching.Click

        '=== 近似色照合 ===
        '照合結果欄に全定義色を近い順に書き出して、そこからお好みの色を選んでもらう。
        '色の距離に、透明度は考慮されない。
        '《ListBox.DataSource》でアイテム追加を行うと、強制的に1番目の項目が選択されてしまうので、今回は使えない。

        Windows.Forms.Cursor.Current = Cursors.WaitCursor

        With lstMatchingResult

            '--- 表示停止 ---
            .BeginUpdate()

            '--- 全定義色との距離を測る ---
            .Items.Clear()
            For Each item As ColorChip In cmbDefinitionColor.Items
                Dim nearness As Double = item.Nearness(DirectCast(picSampleBox.Tag, Color))
                .Items.Add(New ColorBrotherItem With {.ColorName = item.SpecimenColor.Name, .NearnessValue = nearness})
            Next
            ArrayList.Adapter(.Items).Sort(New ColorBrotherSorter) '距離の近い順にソート（《ArrayList》に《ListBox.Items》の内容を関連付けてソートを行う。ソート方法は《ColorBrotherSorter》に記述される。）

            '--- 現在の選択色をハイライト ---
            .SelectedIndex = .FindStringExact(cmbDefinitionColor.Text) '《FindStringExact》メソッドは、《ToString》の文字列と照合する。

            '--- 表示再開 ---
            tabListIndex.SelectTab(tabMatchingResult)
            .EndUpdate()

        End With

    End Sub

    Private Sub nudNearRange_KeyDown(sender As Object, e As System.Windows.Forms.KeyEventArgs) Handles nudNearRange.KeyDown

        If e.KeyCode = Keys.Enter Then
            '=== [Enter]キーで照合開始 ===
            btnMatching.PerformClick()
            e.SuppressKeyPress = False
        End If

    End Sub

    Private Sub CodeSwitch_Click(sender As System.Object, e As System.EventArgs) Handles tsmNameOnly.Click, tsmNamePlus.Click

        '=== プログラム内部表現での表示方法を切り替える ===
        CheckMarkSwitcher(tssbCode.DropDownItems, sender)
        StatusRewrite()

    End Sub

    Private Sub HtmlSwith_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmNatural.Click, tsmLower.Click, tsmUpper.Click

        '=== HTML表現での表示方法を切り替える ===
        CheckMarkSwitcher(tssbHtml.DropDownItems, sender)
        StatusRewrite()

    End Sub

    Private Sub BaseSwitch_Click(sender As System.Object, e As System.EventArgs) Handles tsmBase2.Click, tsmBase8.Click, tsmBase10.Click, tsmBase16.Click

        '=== 数値表現での表示方法を切り替える ===
        CheckMarkSwitcher(tssbNumerical.DropDownItems, sender)
        tsmOK.PerformClick() '直接入力中に奇数を変更したくなった時への対応。
        StatusRewrite()

        '--- 10進数表記時はパレット要素の基数も合わせる ---
        If sender Is tsmBase10 Then
            pltAlpha.nudValue.Hexadecimal = False
            pltRed.nudValue.Hexadecimal = False
            pltGreen.nudValue.Hexadecimal = False
            pltBlue.nudValue.Hexadecimal = False
        Else
            pltAlpha.nudValue.Hexadecimal = True
            pltRed.nudValue.Hexadecimal = True
            pltGreen.nudValue.Hexadecimal = True
            pltBlue.nudValue.Hexadecimal = True
        End If

    End Sub

    Private Sub tssbNumerical_DropDownOpening(sender As System.Object, e As System.EventArgs) Handles tssbNumerical.DropDownOpening

        '=== ARGB値入力テキストボックス初期化 ===
        tstInputARGB.Text = ""

    End Sub

    Private Sub tsmOK_Click(sender As Object, e As System.EventArgs) Handles tsmOK.Click

        '=== 入力されたARGB値をパレットに再現 ===
        'アルファ値のない数値が入力された場合は、パレットで手修正してもらう。

        If tstInputARGB.Text <> "" AndAlso MessageBox.Show("入力されたARGB値をパレットに反映します。", Me.Text, MessageBoxButtons.OKCancel) = Windows.Forms.DialogResult.OK Then

            Try

                '--- 基数取得 ---
                Dim base As Integer
                Select Case True
                    Case tsmBase2.Checked
                        base = 2
                    Case tsmBase8.Checked
                        base = 8
                    Case tsmBase10.Checked
                        base = 10
                    Case tsmBase16.Checked
                        base = 16
                End Select

                '--- 変換 ---
                ColorReplace(Color.FromArgb(Convert.ToInt32(tstInputARGB.Text, base)))

            Catch ex As Exception

                MessageBox.Show("ARGB値が、無効です。", Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)

            End Try

        End If

    End Sub

    Private Sub tsmEdit_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmEdit.DropDownClosed

        If tstFormat.Modified Then

            If MessageBox.Show("入力された書式に変更します。", Me.Text, MessageBoxButtons.OKCancel) = Windows.Forms.DialogResult.OK Then

                '=== 作成結果を反映 ===
                CheckMarkSwitcher(tssbUser.DropDownItems, Nothing) '選択中のプリセットを解除

                FormatDecide()
                StatusRewrite()

            Else

                '=== キャンセル ===
                tstFormat.Text = tstFormat.Tag 'バックアップを戻す。

            End If

            tstFormat.Modified = False

        End If

    End Sub

    Private Sub TextInput_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles tstFormat.KeyDown, tstInputARGB.KeyDown

        Select Case e.KeyCode

            Case Keys.Enter

                '=== メニューを閉じる ===
                '自分の次のアイテム（[OK]または[閉じる]）をクリックしたことにする。
                Dim oneself As ToolStripTextBox = DirectCast(sender, ToolStripTextBox) '自分自身
                Dim parent As ToolStripMenuItem = DirectCast(oneself.OwnerItem, ToolStripMenuItem) '親
                Dim nextItem As ToolStripMenuItem = DirectCast(parent.DropDown.GetNextItem(oneself, ArrowDirection.Down), ToolStripMenuItem) '親のメニューアイテムから自分の次のメニューアイテムを取得する。
                nextItem.PerformClick()

                e.SuppressKeyPress = True

            Case Keys.A

                '=== 全選択 ===
                '標準で全選択が機能しない。
                If e.Control Then
                    DirectCast(sender, ToolStripTextBox).SelectAll()
                    e.SuppressKeyPress = True
                End If

        End Select

    End Sub

    Private Sub PresetItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        'ユーザ書式のプリセット項目のクリックイベントに割り当てられる。

        '=== 選択されたプリセットを有効化 ===
        CheckMarkSwitcher(tssbUser.DropDownItems, sender) 'プリセットのチェックマークを切り替え

        With DirectCast(sender, ToolStripMenuItem)
            tssbUser.ToolTipText = "プリセット：" & .Text & vbCrLf & "※クリックでコピーできます。"
            tstFormat.Text = .Tag
            tstFormat.Tag = .Tag '変更を取り消すときのためにバックアップ
        End With

        StatusRewrite()

    End Sub

    Private Sub StatusButton_ButtonClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tssbCode.ButtonClick, tssbHtml.ButtonClick, tssbNumerical.ButtonClick, tssbUser.ButtonClick

        '=== クリップボードへコピー ===
        Clipboard.SetText(sender.text.Replace(ENTER_STRING, vbCrLf).Replace("&&", "&")) 'ボタン表示用に整形した、"&"と"改行"を元に戻す。

    End Sub

    ''' <summary>
    ''' 色箱付き《ComboBox》,《ListBox》コントロールのアイテム描画
    ''' </summary>
    ''' <param name="Text">表示テキスト</param>
    ''' <param name="BoxColor">色箱の色</param>
    ''' <param name="e">イベント パラメータ</param>
    ''' <remarks></remarks>
    Private Sub DrawColorBoxItem(Text As String, BoxColor As Color, e As System.Windows.Forms.DrawItemEventArgs)

        Dim colorBoxSize As Size '色箱のサイズ
        Dim colorBoxRect As Rectangle '色箱の形状
        Dim captionPoint As Point 'キャプション表示位置

        '=== 背景描画 ===
        e.DrawBackground()

        '=== パラメータ作成 ===
        With e.Bounds
            colorBoxSize = New Size(COLOR_BOX_WIDTH, .Height - COLOR_BOX_MARGIN * 2)
            colorBoxRect = New Rectangle(.X + COLOR_BOX_MARGIN, .Y + (.Height - colorBoxSize.Height) \ 2, colorBoxSize.Width - 1, colorBoxSize.Height - 1)
            captionPoint = New Point(.X + colorBoxSize.Width + COLOR_BOX_MARGIN * 2, .Y + (.Height - e.Font.Height) \ 2)
        End With

        With e.Graphics

            '=== 色箱 ===
            .FillRectangle(New SolidBrush(BoxColor), colorBoxRect)
            .DrawRectangle(Pens.Black, colorBoxRect)

            '=== キャプション ===
            .DrawString(Text, e.Font, New SolidBrush(e.ForeColor), captionPoint)

        End With

        '=== フォーカス描画 ===
        e.DrawFocusRectangle()

    End Sub

    ''' <summary>
    ''' 色コレクションの選択状態を切り替える
    ''' </summary>
    ''' <param name="ChoiceColorChip">選択された色チップ</param>
    ''' <remarks></remarks>
    Private Sub CollectionSelectChange(ByVal ChoiceColorChip As ColorChip)

        With flpColorCollection

            .SuspendLayout() 'ちらつきを抑えるために再描画を一時停止する。

            '=== 以前選択されていた色チップの強調表示を解除する ===
            If .Tag IsNot Nothing Then
                DirectCast(.Tag, ColorChip).Selected = False
                .Tag = Nothing '記憶していた色チップを消す。
            End If

            '=== 新しく選択された色チップを強調表示する ===
            If ChoiceColorChip IsNot Nothing Then
                ChoiceColorChip.Selected = True
                .Tag = ChoiceColorChip '選択された色チップを記憶しておく。
                .ScrollControlIntoView(ChoiceColorChip) '選択された色チップが表示領域内に入るようにスクロールさせる。
            End If

            .ResumeLayout()

        End With

    End Sub

    ''' <summary>
    ''' ステータス書式設定のチェックマークを付け替える
    ''' </summary>
    ''' <param name="MenuObject">対象となる《ToolStripItemCollection》（《DropDownItems》）</param>
    ''' <param name="NewCheckItem">新しくチェックを付ける項目</param>
    ''' <remarks></remarks>
    Private Sub CheckMarkSwitcher(ByRef MenuObject As ToolStripItemCollection, ByRef NewCheckItem As ToolStripMenuItem)

        '全《ToolStripMenuItem》の内、チェックマークが付くのは1つだけが前提。
        'この条件を外すには、検索開始番号と終了番号を指定するとか、対象アイテムを別途コレクション（配列化）して持ってくるしかない。

        For Each item As ToolStripItem In MenuObject
            If item.GetType Is GetType(ToolStripMenuItem) Then '《ToolStripMenuItem》のみを抽出する。
                DirectCast(item, ToolStripMenuItem).Checked = (item Is NewCheckItem)
            End If
        Next

    End Sub

    ''' <summary>
    ''' 変更した書式を確定する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub FormatDecide()

        If tstFormat.Text <> "" Then
            tssbUser.ToolTipText = "ユーザ設定" & vbCrLf & "※クリックでコピーできます。"
        Else
            tstFormat.Text = "{{0}}:改行{0}{{1}}:色名({1})　{{2}}:HTMLカラー({2})　{{3}}:ARGB({3}){0}{{4}}:透明度({4})　{{5}}:赤({5})　{{6}}:青({6})　{{7}}:緑({7}){0}{{8}}:色相({8})　{{9}}:彩度({9})　{{10}}:明度({10}){0}以上"
            tssbUser.ToolTipText = "ユーザ書式：サンプル" & vbCrLf & "書式を作成することができます。"
        End If
        tstFormat.Tag = tstFormat.Text '変更を取り消すときのためにバックアップ

    End Sub

    ''' <summary>
    ''' 色情報を置き換える
    ''' </summary>
    ''' <param name="NewColor">新しく選択した色</param>
    ''' <remarks>全ての色操作が、ここを起点に行われる。</remarks>
    Private Sub ColorReplace(ByVal NewColor As Color)

        Static nowWorking As Boolean = False '作業中フラグ

        '=== 作業開始許可判定 ===
        If nowWorking Then Exit Sub '※※※ 作業中なら抜ける（変更した各コントロールから何度も呼び出されることに対処する）。
        nowWorking = True

        '=== 色リストの変更 ===
        If NewColor.IsNamedColor Then

            '--- 定義色内から選んだ場合 ---
            cmbDefinitionColor.Text = NewColor.ToKnownColor.ToString

            Select Case True
                Case NewColor.IsSystemColor
                    '選択色がシステム色だった
                    cmbWebColor.SelectedIndex = -1
                    cmbSystemColor.Text = cmbDefinitionColor.Text
                    tabListIndex.SelectTab(tabSystem)
                Case Else
                    '選択色がWeb色だった
                    cmbWebColor.Text = cmbDefinitionColor.Text
                    cmbSystemColor.SelectedIndex = -1
                    tabListIndex.SelectTab(tabWeb)
            End Select

            '照合結果に含まれているか確認
            With lstMatchingResult
                .SelectedIndex = .FindStringExact(cmbDefinitionColor.Text) '《FindStringExact》メソッドは、《ToString》の文字列と照合するみたい。
            End With

            '色コレクション切り替え
            CollectionSelectChange(DirectCast(cmbDefinitionColor.SelectedItem, ColorChip))

            '覚え書きリストの解除
            lstMemo.SelectedIndex = -1

        Else

            '--- パレットで調合した場合 ---
            '色リスト解除
            cmbDefinitionColor.SelectedIndex = -1
            cmbSystemColor.SelectedIndex = -1
            cmbWebColor.SelectedIndex = -1
            lstMatchingResult.SelectedIndex = -1
            tabListIndex.SelectTab(tabDefinition)

            '色コレクション解除
            CollectionSelectChange(Nothing)

            '覚え書きリストと照合
            With lstMemo
                .SelectedIndex = .Items.IndexOf(NewColor.ToArgb.ToString("X8"))
            End With

        End If

        '=== パレットの変更 ===
        '--- RGB値 ---
        pltAlpha.ARGB = NewColor
        pltRed.ARGB = NewColor
        pltGreen.ARGB = NewColor
        pltBlue.ARGB = NewColor

        '--- 色サンプル ---
        With picSampleBox
            .Tag = NewColor
            .Refresh()
        End With

        '--- ステータス ---
        StatusRewrite()

        '=== 後片付け ===
        nowWorking = False

    End Sub

    ''' <summary>
    ''' ステータス表示
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub StatusRewrite()

        '=== 現在の色を取得 ===
        If picSampleBox.Tag Is Nothing Then Exit Sub '※※※ 未設定なら無視する。
        Dim nowColor As Color = DirectCast(picSampleBox.Tag, Color) '現在色は、《picSampleBox.Tag》に格納されている。

        '=== プログラム内部表現 ===
        Dim section As String = "" '色の所属先名
        If tsmNamePlus.Checked Then
            'おまけ付き
            If nowColor.IsSystemColor Then
                section = "SystemColors." 'システム色
            ElseIf nowColor.IsKnownColor Then
                section = "Color." 'Web色
            End If
        End If
        tssbCode.Text = section & nowColor.Name

        '=== HTML表現 ===
        Dim html As String = ColorTranslator.ToHtml(nowColor)
        Select Case True
            Case tsmLower.Checked
                tssbHtml.Text = html.ToLower '小文字
            Case tsmUpper.Checked
                tssbHtml.Text = html.ToUpper '大文字
            Case Else
                tssbHtml.Text = html 'そのまま
        End Select

        '=== 数値表現 ===
        Select Case True
            Case tsmBase2.Checked
                tssbNumerical.Text = Convert.ToString(nowColor.ToArgb, 2)
            Case tsmBase8.Checked
                tssbNumerical.Text = Convert.ToString(nowColor.ToArgb, 8)
            Case tsmBase10.Checked
                tssbNumerical.Text = Convert.ToString(nowColor.ToArgb, 10)
            Case tsmBase16.Checked
                tssbNumerical.Text = Convert.ToString(nowColor.ToArgb, 16)
        End Select

        '=== ユーザ書式表現 ===
        Try
            With nowColor
                tssbUser.Text = String.Format(tstFormat.Text, vbCrLf, .Name, html, .ToArgb, .A, .R, .G, .B, CSng(.A / 255), .GetHue, .GetSaturation, .GetBrightness).Replace("&", "&&").Replace(vbCrLf, ENTER_STRING)
                '"&"が一回だけだとショートカットのアンダーラインになってしまう。しかし、編集上は"&"一回の方が自然である。よって、表示する時だけ"&&"へ変換する。
                '改行文字も、代わりの文字列に変換（改行があると、《ToolStrip》の高さが広がってレイアウトが崩れてしまうので）。代替文字列と同じものを書式に含められない。
            End With
        Catch ex As Exception
            tssbUser.Text = "エラー：書式が不正です。"
        End Try

    End Sub

    ''' <summary>
    ''' 覚え書きを変更した
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub MemoChanged()

        If Not MemoChange Then
            MemoChange = True
            grpMemo.Text &= "*"
        End If

    End Sub

    ''' <summary>
    ''' RGBとBGRを相互変換
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function ColorCodeReverses(Code As Integer) As Integer

        Dim element(2) As Integer

        element(0) = (Code And &HFF) << 16     'bit7 - bit0 => bit24 - bit16
        element(1) = Code And &HFF00           'bit15 - bit8
        element(2) = (Code And &HFF0000) >> 16 'bit24 - bit16 => bit7 - bit0

        Return element(0) Or element(1) Or element(2)

    End Function

End Class
