﻿'[FormMain]クラスの続き
'ファイル操作を伴うルーチンをまとまる



Partial Class FormMain

    ''' <summary>覚え書きを保存するファイルの名前</summary>
    Private Const MEMO_FILE_NAME As String = "memo.dat"

    ''' <summary>プリセット書式のファイル名</summary>
    Private Const PRESET_FILE_NAME As String = "preset.csv"



    '============================================================================================



    ''' <summary>
    ''' 「覚え書き」を読み込む
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub MemoLoad()

        Dim path As String = System.IO.Path.Combine(My.Application.Info.DirectoryPath, MEMO_FILE_NAME) '保存先のファイル名

        Try

            If System.IO.File.Exists(path) Then

                Windows.Forms.Cursor.Current = Cursors.WaitCursor

                '=== ファイルの中身を全て読み込む ===
                Dim lines As String() = System.IO.File.ReadAllLines(path, System.Text.Encoding.GetEncoding("shift_jis"))

                '=== 読み込んだアイテムを復元 ===
                Dim errorMessage As String = "" 'エラーメッセージ

                lstMemo.BeginUpdate()

                For Each item As String In lines
                    Try
                        If item <> "" Then
                            lstMemo.Items.Add(Convert.ToInt32(item, 16).ToString("X8")) '読み込んだ文字列を、一旦Integer型に変換して（有効なデータかを[Try]ブロックで調べる））、16進数8桁に再度戻す。
                        End If
                    Catch ex As Exception
                        errorMessage &= String.Format("""{0}""・・・{1}", item, ex.Message) & vbCrLf
                    End Try
                Next

                lstMemo.EndUpdate()

                '=== エラーメッセージ表示 ===
                If errorMessage <> "" Then MessageBox.Show(String.Format("「覚え書き」の復元中にエラーが発生しました。読み飛ばして起動を継続します。{0}※記述は、1行1色, Integer型(32bit), 16進数表記でなければなりません。{0}{0}{1}", vbCrLf, errorMessage), Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)

            End If

        Catch ex As Exception

            MessageBox.Show("「覚え書き」の読み込み中に予期しないエラーが発生しました。" & vbCrLf & vbCrLf & ex.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)

        End Try

    End Sub

    ''' <summary>
    ''' 「覚え書き」を保存する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub MemoSave()

        Try

            Windows.Forms.Cursor.Current = Cursors.WaitCursor

            '=== 登録内容を1行ずつ書き出す ===
            Using writer As New System.IO.StreamWriter(System.IO.Path.Combine(My.Application.Info.DirectoryPath, MEMO_FILE_NAME), _
                                                       False, _
                                                       System.Text.Encoding.GetEncoding("shift_jis"))
                For Each item As String In lstMemo.Items
                    writer.WriteLine(item)
                Next
            End Using

        Catch ex As Exception

            MessageBox.Show("「覚え書き」の保存中にエラーが発生しました。" & vbCrLf & vbCrLf & ex.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
            Throw '発生したエラーを呼び出し元に返す。

        End Try

    End Sub

    ''' <summary>
    ''' ユーザ書式のプリセットを読み込む
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub PresetLoad()

        '=== メニュー登録 ===
        tssbUser.DropDown.SuspendLayout()

        Dim path As String = System.IO.Path.Combine(My.Application.Info.DirectoryPath, PRESET_FILE_NAME) 'パス作成
        Dim defaultItem As ToolStripMenuItem = Nothing '初期書式にするアイテム

        Try

            If System.IO.File.Exists(path) Then

                'CSVのフィールド構成・・・0:フラグ(/=注釈,!=初期書式,-=通常) / 1:テキスト / 2:書式 / 3:ツールチップ
                Using csvReader As New FileIO.TextFieldParser(path, System.Text.Encoding.GetEncoding("shift_jis")) With {.Delimiters = New String() {","}} 'CSV読み取り機を生成して、ついでにフィールド区切り文字もしてしとく。

                    Do Until csvReader.EndOfData

                        Dim fields As String() = csvReader.ReadFields '1レコードを読み込んでフィールドに自動分割

                        If fields(0) <> "/" Then
                            '各フィールドからメニューアイテムを作成して登録する
                            Dim newItem As New ToolStripMenuItem(fields(1), Nothing, New EventHandler(AddressOf PresetItem_Click)) With {.Tag = fields(2), _
                                                                                                                                         .ToolTipText = fields(3)}
                            tssbUser.DropDownItems.Add(newItem)

                            If fields(0) = "!" Then defaultItem = newItem '初期書式の抽出
                        End If

                    Loop

                End Using

            End If

        Catch ex As Exception

            MessageBox.Show("「プリセット」の読み込み中にエラーが発生しました。" & vbCrLf & vbCrLf & ex.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)

        End Try

        tssbUser.DropDown.ResumeLayout()

        '=== 初期書式を設定 ===
        If defaultItem Is Nothing Then

            '--- 指定なし ---
            Dim index As Integer = tssbUser.DropDownItems.IndexOf(tssPreset) + 1 '区切り線の次のアイテム番号を取得
            If tssbUser.DropDownItems.Count > index Then
                tssbUser.DropDownItems(index).PerformClick() 'プリセットがある
            Else
                FormatDecide() 'プリセットが1つもない
                tssPreset.Visible = False
            End If

        Else

            '--- 指定あり ---
            defaultItem.PerformClick() 'アイテムをクリックしたことにして有効化

        End If

    End Sub

End Class
