﻿''' <summary>
''' パレット(ARGB)要素
''' </summary>
''' <remarks>ユーザー コントロール</remarks>
Public Class PalletElement

    ''' <summary>
    ''' 担当するパレット要素の番号
    ''' </summary>
    ''' <remarks></remarks>
    Public Enum PartNo As Byte

        ''' <summary>透明度</summary>
        Alpha = 1

        ''' <summary>赤</summary>
        Red

        ''' <summary>緑</summary>
        Green

        ''' <summary>青</summary>
        Blue

    End Enum



    '============================================================================================



#Region "イベント"

    ''' <summary>
    ''' 値の変更
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    <System.ComponentModel.Category("プロパティ変更")> <System.ComponentModel.Description("値が変更されたときに発生します。")> _
    Public Event ValueChanged(ByVal sender As Object, ByVal e As EventArgs)

#End Region



    '============================================================================================



#Region "プロパティ"

    Private _ARGB As Color
    Private _Part As PartNo


    '--------------------------------------------------------------------------------------------


    ''' <summary>
    ''' 現在の色
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.ComponentModel.Category("表示")> <System.ComponentModel.Description("現在の色を取得・設定する。")> _
    Public Property ARGB() As Color

        Get
            Return _ARGB
        End Get

        Set(ByVal value As Color)

            If _ARGB <> value Then

                _ARGB = value

                '=== 設定値を反映 ===
                trkAdjust.Value = Choose(Part, _ARGB.A, _ARGB.R, _ARGB.G, _ARGB.B)
                nudValue.Value = trkAdjust.Value
                PicSampleBar.Refresh()

                '=== イベント発生 ===
                RaiseEvent ValueChanged(Me, New EventArgs)

            End If

        End Set

    End Property

    ''' <summary>
    ''' 担当する色要素
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.ComponentModel.Category("動作")> <System.ComponentModel.Description("担当する色要素を取得・設定する。")> <System.ComponentModel.DefaultValue(PartNo.Alpha)> _
    Public Property Part() As PartNo

        Get
            Return _Part
        End Get

        Set(ByVal value As PartNo)

            _Part = value

            Select Case _Part

                Case PartNo.Alpha
                    '=== 透明 ===
                    With lblPartName
                        .Text = "A"
                        .ForeColor = Color.Black
                    End With
                    nudValue.Value = ARGB.A

                Case PartNo.Red
                    '=== 赤 ===
                    With lblPartName
                        .Text = "R"
                        .ForeColor = Color.Red
                    End With
                    nudValue.Value = ARGB.R

                Case PartNo.Green
                    '=== 緑 ===
                    With lblPartName
                        .Text = "G"
                        .ForeColor = Color.Green
                    End With
                    nudValue.Value = ARGB.G

                Case PartNo.Blue
                    '=== 青 ===
                    With lblPartName
                        .Text = "B"
                        .ForeColor = Color.Blue
                    End With
                    nudValue.Value = ARGB.B

            End Select

        End Set

    End Property

    <System.ComponentModel.Category("表示")> <System.ComponentModel.Description("背景色を取得・設定する。")> _
    Public Property BackgroundColor() As Color
        Get
            Return PicSampleBar.BackColor
        End Get
        Set(ByVal value As Color)
            PicSampleBar.BackColor = value
        End Set
    End Property

#End Region



    '============================================================================================



    Public Sub New()

        ' この呼び出しは、Windows フォーム デザイナで必要です。
        InitializeComponent()

        ' InitializeComponent() 呼び出しの後で初期化を追加します。
        ARGB = Color.FromArgb(0, 0, 0, 0) '無色透明
        Part = PartNo.Alpha
        BackgroundColor = Color.Black

    End Sub

    Private Sub trkAdjust_ValueChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles trkAdjust.ValueChanged

        '=== 値が変更された ===
        Color_Make(DirectCast(sender, TrackBar).Value)

    End Sub

    Private Sub nudValue_ValueChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles nudValue.ValueChanged

        '=== 値が変更された ===
        Color_Make(DirectCast(sender, NumericUpDown).Value)

    End Sub

    Private Sub PicSampleBar_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles PicSampleBar.MouseDown

        '=== サンプルバーから選択 ===
        Color_Make(Int((e.X + 1) / sender.Width * 256 + 0.9) - 1)

    End Sub

    Private Sub PicSampleBar_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles PicSampleBar.Resize

        '=== サイズ変化に伴う再描画 ===
        DirectCast(sender, PictureBox).Refresh()

    End Sub

    Private Sub PicSampleBar_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles PicSampleBar.Paint

        '=== サンプルバー描画 ===
        Dim startColor, endColor As Color 'グラデーションの開始色と終了色
        With ARGB
            Select Case Part
                Case PartNo.Alpha
                    startColor = Color.FromArgb(0, .R, .G, .B)
                    endColor = Color.FromArgb(255, .R, .G, .B)
                Case PartNo.Red
                    startColor = Color.FromArgb(.A, 0, .G, .B)
                    endColor = Color.FromArgb(.A, 255, .G, .B)
                Case PartNo.Green
                    startColor = Color.FromArgb(.A, .R, 0, .B)
                    endColor = Color.FromArgb(.A, .R, 255, .B)
                Case Else
                    startColor = Color.FromArgb(.A, .R, .G, 0)
                    endColor = Color.FromArgb(.A, .R, .G, 255)
            End Select
        End With

        With DirectCast(sender, PictureBox)
            Dim gb As New Drawing2D.LinearGradientBrush(.ClientRectangle, startColor, endColor, Drawing2D.LinearGradientMode.Horizontal) '線形グラデーションブラシ作製
            e.Graphics.FillRectangle(gb, .ClientRectangle) 'グラデーション描画
            '※グラデーション描画機能の都合上、サンプルバーの色とトラックバーの数値は必ずしも一致しない（終了色が1ドット分足りない）。 → なので、あくまで目安にしかならない。
        End With

    End Sub

    ''' <summary>
    ''' 色生成
    ''' </summary>
    ''' <param name="Value">担当要素の値</param>
    ''' <remarks></remarks>
    Private Sub Color_Make(ByVal Value As Integer)

        '=== 現在の調整値を反映させた色を生成する ===
        Select Case Part
            Case PartNo.Alpha
                ARGB = Color.FromArgb(Value, ARGB.R, ARGB.G, ARGB.B)
            Case PartNo.Red
                ARGB = Color.FromArgb(ARGB.A, Value, ARGB.G, ARGB.B)
            Case PartNo.Green
                ARGB = Color.FromArgb(ARGB.A, ARGB.R, Value, ARGB.B)
            Case PartNo.Blue
                ARGB = Color.FromArgb(ARGB.A, ARGB.R, ARGB.G, Value)
        End Select

    End Sub

End Class
